#include "stdafx.h"

#include "FieldPatch.h"

// FieldPatch

CreateFrameOperation<FieldPatch, &FieldPatch::readFieldPatch> g_createFieldPatch("FieldPatch");
CreateFrameOperation<FieldPatch, &FieldPatch::readFieldPatch2> g_createFieldPatch2("FieldPatch2");

FieldPatch::FieldPatch(CVclipDoc& owner, 
	LONG aSourceFrame, LONG aSourceOrder, int aSourceX, int aSourceY, int aSourceWidth, int aSourceHeight,
	LONG aTargetFrame, LONG aTargetOrder, int aTargetX, int aTargetY, int aTargetWidth, int aTargetHeight,
	HOverlay mode)
: PatchOperationWithSource(owner,
		aSourceFrame, aSourceX, aSourceY, aSourceWidth, aSourceHeight,
		aTargetFrame, aTargetX, aTargetY, aTargetWidth, aTargetHeight),
	m_sourceOrder(aSourceOrder), m_targetOrder(aTargetOrder),
	m_mode(mode)
{
}

LONG FieldPatch::sourceOrder() const
{
	return m_sourceOrder;
}
LONG FieldPatch::targetOrder() const
{
	return m_targetOrder;
}

HOverlay FieldPatch::mode() const
{
	return m_mode;
}
void FieldPatch::setMode(HOverlay mode)
{
	m_mode = mode;
	onModified();
}

bool FieldPatch::usesFrame(LONG frame) const
{
	return sourceFrame() == frame;
}

const char* FieldPatch::typeCode() const
{
	return "FieldPatch2";
}

void FieldPatch::writeTo(std::ostream& stream) const
{
	stream             <<
		sourceFrame()    << " " <<
		m_sourceOrder    << " " <<
		sourceX()        << " " <<
		sourceY()        << " " <<
		sourceWidth()    << " " <<
		sourceHeight()   << " " <<
		targetFrame()    << " " <<
		m_targetOrder    << " " <<
		targetX()        << " " <<
		targetY()        << " " <<
		targetWidth()    << " " <<
		targetHeight()   << " " <<
		m_mode;
}

void FieldPatch::readFieldPatch(std::istream& stream)
{
	CSize size;
	CPoint sourceOrigin, targetOrigin;
	LONG sourceFrame, targetFrame;
	stream             >>
		sourceFrame      >>
		m_sourceOrder    >>
		sourceOrigin.x   >>
		sourceOrigin.y   >>
		size.cx          >>
		size.cy          >>
		targetFrame      >>
		m_targetOrder    >>
		targetOrigin.x   >>
		targetOrigin.y   >>
		m_mode;
	setSourceFrame(sourceFrame);
	setTargetFrame(targetFrame);
	moveSourceTo(sourceOrigin);
	moveTargetTo(targetOrigin);
	resizeSource(size);
	resizeTarget(size);
}

void FieldPatch::readFieldPatch2(std::istream& stream)
{
	CSize sourceSize, targetSize;
	CPoint sourceOrigin, targetOrigin;
	LONG sourceFrame, targetFrame;
	stream             >>
		sourceFrame      >>
		m_sourceOrder    >>
		sourceOrigin.x   >>
		sourceOrigin.y   >>
		sourceSize.cx    >>
		sourceSize.cy    >>
		targetFrame      >>
		m_targetOrder    >>
		targetOrigin.x   >>
		targetOrigin.y   >>
		targetSize.cx    >>
		targetSize.cy    >>
		m_mode;
	setSourceFrame(sourceFrame);
	setTargetFrame(targetFrame);
	moveSourceTo(sourceOrigin);
	moveTargetTo(targetOrigin);
	resizeSource(sourceSize);
	resizeTarget(targetSize);
}

void FieldPatch::accept(IOperationVisitor& visitor)
{
	return visitor(*this);
}

