#include "stdafx.h"

#include "FramePatch.h"


// FramePatch

CreateFrameOperation<FramePatch, &FramePatch::readFramePatch> g_createFramePatch("FramePatch");
CreateFrameOperation<FramePatch, &FramePatch::readFramePatch2> g_createFramePatch2("FramePatch2");

FramePatch::FramePatch(CVclipDoc& owner, 
	LONG aSourceFrame, int aSourceX, int aSourceY, int aSourceWidth, int aSourceHeight,
	LONG aTargetFrame, int aTargetX, int aTargetY, int aTargetWidth, int aTargetHeight, HOverlay mode)
: PatchOperationWithSource(owner, 
		aSourceFrame, aSourceX, aSourceY, aSourceWidth, aSourceHeight,
		aTargetFrame, aTargetX, aTargetY, aTargetWidth, aTargetHeight),
	m_mode(mode)
{
}

HOverlay FramePatch::mode() const
{
	return m_mode;
}

void FramePatch::setMode(HOverlay mode)
{
	m_mode = mode;
	onModified();
}

bool FramePatch::usesFrame(LONG frame) const
{
	return sourceFrame() == frame;
}

const char* FramePatch::typeCode() const
{
	return "FramePatch2";
}

void FramePatch::writeTo(std::ostream& stream) const
{
	stream             <<
		sourceFrame()    << " " <<
		sourceX()        << " " <<
		sourceY()        << " " <<
		sourceWidth()    << " " <<
		sourceHeight()   << " " <<
		targetFrame()    << " " <<
		targetX()        << " " <<
		targetY()        << " " <<
		targetWidth()    << " " <<
		targetHeight()   << " " <<
		m_mode;
}

void FramePatch::readFramePatch(std::istream& stream)
{
	LONG sourceFrame, targetFrame;
	CPoint sourceOrigin, targetOrigin;
	CSize size;
	stream              >>
		sourceFrame       >>
		sourceOrigin.x    >>
		sourceOrigin.y    >>
		size.cx           >>
		size.cy           >>
		targetFrame       >>
		targetOrigin.x    >>
		targetOrigin.y    >>
		m_mode;
	setSourceFrame(sourceFrame);
	setTargetFrame(targetFrame);
	moveSourceTo(sourceOrigin);
	moveTargetTo(targetOrigin);
	resizeSource(size);
	resizeTarget(size);
}

void FramePatch::readFramePatch2(std::istream& stream)
{
	LONG sourceFrame, targetFrame;
	CPoint sourceOrigin, targetOrigin;
	CSize sourceSize, targetSize;
	stream              >>
		sourceFrame       >>
		sourceOrigin.x    >>
		sourceOrigin.y    >>
		sourceSize.cx     >>
		sourceSize.cy     >>
		targetFrame       >>
		targetOrigin.x    >>
		targetOrigin.y    >>
		targetSize.cx     >>
		targetSize.cy     >>
		m_mode;
	setSourceFrame(sourceFrame);
	setTargetFrame(targetFrame);
	moveSourceTo(sourceOrigin);
	moveTargetTo(targetOrigin);
	resizeSource(sourceSize);
	resizeTarget(targetSize);
}

void FramePatch::accept(IOperationVisitor& visitor)
{
	return visitor(*this);
}


