#include "stdafx.h"
#include "MediaTime.h"

bool parseMediaTime(const char* s, LONG& milliseconds)
{
	if (!s || !*s) return false; // must not be empty
	LONG hours = 0, minutes = 0, seconds = 0, milli = 0;
	for (; *s && '.' != *s; ++s)
	{
		switch (*s)
		{
		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			seconds = seconds * 10 + (*s - '0');
			break;
		case ':':
			if (hours != 0) return false; // too many components
			hours = minutes;
			minutes = seconds;
			seconds = 0;
			break;
		case '.':
			break;
		default:
			return false; // invalid character
		}
	}
	if ('.' == *s)
	{
		LONG multiplier = 100;
		for (++s; *s; ++s)
		{
			if ('0' <= *s && *s <= '9')
			{
				milli += multiplier * (*s - '0');
				multiplier /= 10;
			}
			else return false; // invalid character
		}
	}

	milliseconds = 
		milli + 1000 * (
		seconds + 60 * (
		minutes + 60 * hours));
	return true;
}

CString formatMediaTime(LONG milliseconds)
{
	CString result;
	result.Format("%d:%02d:%02d.%03d", 
		milliseconds / (1000 * 60 * 60),
		milliseconds / (1000 * 60) % 60,
		milliseconds / 1000 % 60,
		milliseconds % 1000);
	return result;
}

LONG mediaTimeFromFrame(LONG frame, LONG millifps)
{
	return LONG(LONGLONG(frame) * 1000000 / millifps);
}

LONG frameFromMediaTime(LONG milliseconds, LONG millifps)
{
	return LONG(LONGLONG(milliseconds) * millifps / 1000000);
}

void PASCAL DDX_MediaTime(CDataExchange* pDX, int nIDC, LONG& milliseconds)
{
	pDX->PrepareEditCtrl(nIDC);
	HWND hWndCtrl;
	pDX->m_pDlgWnd->GetDlgItem(nIDC, &hWndCtrl);
	
	if (pDX->m_bSaveAndValidate)
	{
		int nLen = ::GetWindowTextLength(hWndCtrl);
		
		CString strTemp;
		::GetWindowText(hWndCtrl, strTemp.GetBufferSetLength(nLen), nLen+1);
		strTemp.ReleaseBuffer();

		if (!parseMediaTime(strTemp, milliseconds))
		{
			CString error;
			error.Format("Please enter a media time.", strTemp);
			AfxMessageBox(error, MB_OK | MB_ICONWARNING);
			pDX->Fail();
		}
	}
	else
	{
		CString strTemp = formatMediaTime(milliseconds);
		::SetWindowText(hWndCtrl, strTemp);
	}
}

void PASCAL DDV_MediaTime(CDataExchange* pDX, LONG milliseconds, LONG minVal, LONG maxVal)
{
	if (milliseconds < minVal || milliseconds > maxVal)
	{
		CString error;
		error.Format("Please enter a media time between %s and %s.", formatMediaTime(minVal).GetBuffer(), formatMediaTime(maxVal).GetBuffer());
		AfxMessageBox(error, MB_OK | MB_ICONWARNING);
		pDX->Fail();
	}
}

