#pragma once

#include "DrawDib.h"
#include "FrameCache.h"
#include "FrameOperation.h"
#include "OverlayMode.h"

#include <atltypes.h>
#include <afxwin.h>

class ViewMode
{
public: // coordinate transforms
	virtual CPoint viewFromFrame(const CPoint& pt) const = 0;
	virtual CPoint frameFromView(const CPoint& pt) const = 0;
	CRect viewFromFrame(const CRect& rect) const;
	CRect frameFromView(const CRect& rect) const;

public: // frame navigation
	virtual LONG next(LONG& frame, LONG& order) const = 0;
	virtual LONG previous(LONG& frame, LONG& order) const = 0;
	virtual bool hasNext(LONG frame, LONG order, LONG frames) const = 0;
	virtual bool hasPrevious(LONG frame, LONG order) const = 0;

	virtual CString formatPosition(LONG frame, LONG order, LONG frames) const = 0;

	virtual bool visible(LONG currentOrder, LONG targetOrder) const = 0;

public: // drawing
	virtual void draw(CDC* pDC, const DrawDib& drawDib, const PFrame& frame, LONG order) const = 0; 

public: // pasting
	virtual PFrameOperation paste(CVclipDoc& document, const struct Clipboard& clipboard, 
		LONG frame, LONG order, HOverlay mode) const = 0;
};

class FrameMode : public ViewMode
{
public:
	CPoint viewFromFrame(const CPoint& point) const;
	CPoint frameFromView(const CPoint& point) const;
	LONG next(LONG& frame, LONG& order) const;
	LONG previous(LONG& frame, LONG& order) const;
	bool hasNext(LONG frame, LONG order, LONG frames) const;
	bool hasPrevious(LONG frame, LONG order) const;
	CString formatPosition(LONG frame, LONG order, LONG frames) const;
	bool visible(LONG currentOrder, LONG targetOrder) const;
	void draw(CDC* pDC, const DrawDib& drawDib, const PFrame& frame, LONG order) const;
	PFrameOperation paste(CVclipDoc& document, const Clipboard& clipboard, LONG frame, LONG order, HOverlay mode) const;
};

class FieldMode : public ViewMode
{
public:
	CPoint viewFromFrame(const CPoint& point) const;
	CPoint frameFromView(const CPoint& point) const;
	using ViewMode::viewFromFrame;
	using ViewMode::frameFromView;
	LONG next(LONG& frame, LONG& order) const;
	LONG previous(LONG& frame, LONG& order) const;
	bool hasNext(LONG frame, LONG order, LONG frames) const;
	bool hasPrevious(LONG frame, LONG order) const;
	CString formatPosition(LONG frame, LONG order, LONG frames) const;
	bool visible(LONG currentOrder, LONG targetOrder) const;
	void draw(CDC* pDC, const DrawDib& drawDib, const PFrame& frame, LONG order) const;
	PFrameOperation paste(CVclipDoc& document, const Clipboard& clipboard, LONG frame, LONG order, HOverlay mode) const;
};

extern FrameMode frameMode;
extern FieldMode fieldMode;

struct Clipboard
{
	const ViewMode* mode;
	LONG frame;
	LONG order;
	CRect selection;

	void copy(const ViewMode* aMode, LONG aFrame, LONG anOrder, const CRect& aSelection);
	PFrameOperation paste(CVclipDoc& document, const ViewMode* aMode, LONG aFrame, LONG anOrder,
		HOverlay overlayMode) const;
	bool empty() const;
};

